package com.mojang.text2speech;

import com.google.common.collect.Queues;
import com.sun.jna.Native;
import com.sun.jna.NativeLibrary;
import com.sun.jna.WString;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.Queue;

public class NarratorWindows implements Narrator {
    // JNA SPECIFIC CODE
    private static boolean libraryFound = false;
    private static final Logger LOGGER = LogManager.getLogger();

    static {
        String result = "";

        try {
            Native.register(SAPIWrapperSolutionDLL.class, NativeLibrary.getInstance("SAPIWrapper_x64"));
            libraryFound = true;
            LOGGER.info("Narrator library for x64 successfully loaded");
        } catch (final UnsatisfiedLinkError e) {
            result += "ERROR : Couldn't load Narrator library : " + e.getMessage() + "\n";
        } catch (final Throwable e) {
            result += "ERROR : Generic error while loading narrator : " + e.getMessage() + "\n";
        }

        if (!libraryFound) {
            try {
                Native.register(SAPIWrapperSolutionDLL.class, NativeLibrary.getInstance("SAPIWrapper_x86"));
                libraryFound = true;
                LOGGER.info("Narrator library for x86 successfully loaded");
            } catch (final UnsatisfiedLinkError e) {
                result += "ERROR : Couldn't load Narrator library : " + e.getMessage() + "\n";
            } catch (final Throwable e) {
                result += "ERROR : Generic error while loading narrator : " + e.getMessage() + "\n";
            }
        }

        if (!libraryFound) {
            LOGGER.warn(result);
        }
    }

    private static class SAPIWrapperSolutionDLL {
        public static native long say(WString msg);
    }
    // END OF JNA SPECIFIC CODE

    private final NarratorThread narratorThread = new NarratorThread();

    public NarratorWindows() {
        final Thread thread = new Thread(narratorThread);
        thread.setDaemon(true);
        thread.setName("Narrator");
        thread.start();
    }

    private static class NarratorThread implements Runnable {
        protected final Queue<String> msgs = Queues.newConcurrentLinkedQueue();

        @Override
        public void run() {
            while (true) {
                if (msgs.peek() != null) {
                    say(msgs.poll());
                }

                try {
                    Thread.sleep(100);
                } catch (final InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

        public void add(final String msg) {
            msgs.add(msg);
        }

        public void clear() {
            msgs.clear();
        }

        private void say(final String text) {
            SAPIWrapperSolutionDLL.say(new WString(text.replaceAll("[<>]", "")));
        }
    }

    @Override
    public void say(final String msg) {
        narratorThread.add(msg);
    }

    @Override
    public void clear() {
        narratorThread.clear();
    }

    @Override
    public boolean active() {
        return libraryFound;
    }
}